 /*
 * Copyright (c) CompanyNameMagicTag 2021-2022. All rights reserved.
 * Description: Huanglong USB common header file
 * Author: AuthorNameMagicTag
 * Create: 2021.06.10
 */

#ifndef _HUANGLONG_USB_H_
#define _HUANGLONG_USB_H_

#include <linux/usb.h>

enum huanglong_usb_state {
	HUANGLONG_USB_STATE_UNKNOWN = 0,
	HUANGLONG_USB_STATE_OFF,
	HUANGLONG_USB_STATE_HOST,
	HUANGLONG_USB_STATE_DEVICE,
};

/**
 * event types passed to huanglong_usb_otg_event().
 */
enum huanglong_usb_event_type {
	SWITCH_TO_HOST = 0,
	SWITCH_TO_DEVICE,
	NONE_EVENT,
};

/* size of struct huanglong_usb_event must be a power of 2 for kfifo */
#define SIZE_HUANGLONG_USB_EVENT 32
struct huanglong_usb_event {
	enum huanglong_usb_event_type type;
	u32 param1;
	u32 param2;
	u32 flags;
	void (*callback)(struct huanglong_usb_event *event);
	void *content;
};

/**
 * Queue a event to be processed.
 * @evnet: the event to be processed.
 *
 * The event will be added to tail of a queue, and processed in a work.
 *
 * Return: 0 means the event added sucessfully. others means event was rejected.
 */
int huanglong_usb_queue_event(struct huanglong_usb_event *event);
int huanglong_usb_otg_event(enum huanglong_usb_event_type);

#endif /* _HUANGLONG_USB_H_ */
